@@ -119,8 +119,8 @@ pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
 {
   PangoCairoATSUIFont *cafont = (PangoCairoATSUIFont *) font;
   PangoATSUIFont *afont = (PangoATSUIFont *) font;
-  CGFontRef cg_font;
-  CTFontRef ct_font;
+  ATSFontRef ats_font;
+  ATSFontMetrics ats_metrics;
   PangoFontMetrics *metrics;
   PangoFontDescription *font_desc;
   PangoLayout *layout;
@@ -128,19 +128,19 @@ pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
   PangoLanguage *language = pango_context_get_language (context);
   const char *sample_str = pango_language_get_sample_string (language);
 
-  cg_font = pango_atsui_font_get_cgfont (afont);
-  ct_font = CTFontCreateWithGraphicsFont(cg_font, cafont->size, NULL, NULL);
-
   metrics = pango_font_metrics_new ();
 
-  metrics->ascent = CTFontGetAscent(ct_font) * PANGO_SCALE;
-  metrics->descent = CTFontGetDescent(ct_font) * PANGO_SCALE;
+  ats_font = pango_atsui_font_get_atsfont (afont);
+  ATSFontGetHorizontalMetrics (ats_font, kATSOptionFlagsDefault, &ats_metrics);
 
-  metrics->underline_position = CTFontGetUnderlinePosition(ct_font) * PANGO_SCALE;
-  metrics->underline_thickness = CTFontGetUnderlineThickness(ct_font) * PANGO_SCALE;
+  metrics->ascent = ats_metrics.ascent * cafont->size * PANGO_SCALE;
+  metrics->descent = -ats_metrics.descent * cafont->size * PANGO_SCALE;
+
+  metrics->underline_position = ats_metrics.underlinePosition * cafont->size * PANGO_SCALE;
+  metrics->underline_thickness = ats_metrics.underlineThickness * cafont->size * PANGO_SCALE;
 
   metrics->strikethrough_position = metrics->ascent / 3;
-  metrics->strikethrough_thickness = metrics->underline_thickness * PANGO_SCALE;
+  metrics->strikethrough_thickness = ats_metrics.underlineThickness * cafont->size * PANGO_SCALE;
 
   layout = pango_layout_new (context);
   font_desc = pango_font_describe_with_absolute_size ((PangoFont *) font);
@@ -155,8 +155,6 @@ pango_cairo_atsui_font_create_metrics_for_context (PangoCairoFont *font,
 
   pango_font_description_free (font_desc);
   g_object_unref (layout);
-  
-  
 
   return metrics;
 }
@@ -270,6 +268,7 @@ _pango_cairo_atsui_font_new (PangoCairoATSUIFontMap     *cafontmap,
 
   size = (double) pango_font_description_get_size (desc) / PANGO_SCALE;
   _pango_atsui_font_set_cgfont (afont, font_id);
+  _pango_atsui_font_set_atsfont (afont, font_ref);
 
   if (context)
     {
